"use strict";

/* exported logger */

var logger = require('../../logger');

var Promise = require("bluebird");

var IdentifiedElement = require('../data/IdentifiedElement');
var ObjectWithListeners = require('../../ObjectWithListeners');

var ServerConnector = require('../../ServerConnector');

function AbstractDbOverlay(params) {
  // call super constructor
  ObjectWithListeners.call(this);

  if (params.map === undefined) {
    throw new Error("map param must be defined");
  }

  this.setName(params.name);
  this.setMap(params.map);

  this.setAllowSearchById(params.allowSearchById);
  this.setAllowGeneralSearch(params.allowGeneralSearch);

  this.setIconType(params.iconType);
  this.setIconStart(params.iconColorStart);

  this._elementsByQuery = [];
  this._elementsByQuery[""] = [];
  this.registerListenerType('onSearch');
  this.registerListenerType('onClear');
  this.registerListenerType('onTargetVisibilityChange');

  this.setQueries([]);
}

AbstractDbOverlay.prototype = Object.create(ObjectWithListeners.prototype);
AbstractDbOverlay.prototype.constructor = AbstractDbOverlay;

AbstractDbOverlay.QueryType = {
  SEARCH_BY_COORDINATES : "SEARCH_BY_COORDINATES",
  SEARCH_BY_TARGET : "SEARCH_BY_TARGET",
  SEARCH_BY_QUERY : "SEARCH_BY_QUERY",
};

AbstractDbOverlay.prototype.encodeQuery = function(type, arg0, arg1, arg2) {
  if (type === AbstractDbOverlay.QueryType.SEARCH_BY_COORDINATES) {
    var modelId = arg0;
    var coordinates = arg1;
    var zoom = arg2;
    return JSON.stringify({
      type : type,
      modelId : modelId,
      coordinates : coordinates,
      zoom : zoom,
    });
  } else if (type === AbstractDbOverlay.QueryType.SEARCH_BY_TARGET) {
    var target = arg0;
    return JSON.stringify({
      type : type,
      target : target,
    });
  } else if (type === AbstractDbOverlay.QueryType.SEARCH_BY_QUERY) {
    var query = arg0;
    var perfect = arg1;
    return JSON.stringify({
      type : type,
      query : query,
      perfect : perfect,
    });
  } else {
    throw new Error("Unknown query type: " + type);
  }
};

AbstractDbOverlay.prototype.decodeQuery = function(query) {
  return JSON.parse(query);
};

AbstractDbOverlay.prototype.searchByQuery = function(originalQuery, perfect, fitBounds) {
  var self = this;
  var query = self.encodeQuery(AbstractDbOverlay.QueryType.SEARCH_BY_QUERY, originalQuery, perfect);
  ServerConnector.getSessionData().setQuery({
    type : self.getName(),
    query : query
  });

  var queries = self.splitQuery(originalQuery);

  var encodedQueries = [];
  var promises = [];
  for (var i = 0; i < queries.length; i++) {
    encodedQueries.push(self.encodeQuery(AbstractDbOverlay.QueryType.SEARCH_BY_QUERY, queries[i], perfect));
    promises.push(self.searchBySingleQuery(queries[i], perfect));
  }

  var res;
  return Promise.all(promises).then(function(results) {
    self.setQueries(encodedQueries);
    res = results;
    return self.callListeners('onSearch', {
      fitBounds : fitBounds,
      identifiedElements : res,
      type : AbstractDbOverlay.QueryType.SEARCH_BY_QUERY,
    });
  }).then(function() {
    return res;
  });
};

AbstractDbOverlay.prototype.searchNamesByTarget = function(element) {
  var self = this;
  var query = self.encodeQuery(AbstractDbOverlay.QueryType.SEARCH_BY_TARGET, element);

  if (self._elementsByQuery[query] !== undefined) {
    return Promise.resolve(self._elementsByQuery[query]);
  } else {
    return self.getNamesByTargetFromServer({
      target : element
    }).then(function(drugNames) {
      self._elementsByQuery[query] = drugNames;
      return self._elementsByQuery[query];
    });
  }
};

AbstractDbOverlay.prototype.setQueries = function(queries) {
  this._queries = queries;
};

AbstractDbOverlay.prototype.getQueries = function() {
  return this._queries;
};

AbstractDbOverlay.prototype.createIdentifiedElementsForTargetingClass = function(targetingElement, colourId) {
  var self = this;
  var result = [];
  var iconCounter = 1;
  var targets = targetingElement.getTargets();
  for (var k = 0; k < targets.length; k++) {
    var target = targets[k];
    if (target.isVisible()) {
      var elements = target.getTargetElements();
      for (var l = 0; l < elements.length; l++) {
        var element = elements[l];
        var ie = new IdentifiedElement(element);
        if (element.getType() === "ALIAS") {
          ie.setIcon(self.getIcon(colourId, iconCounter));
        } else if (element.getType() !== "REACTION") {
          throw new Error("Unknown element type: " + element.getType());
        }
        result.push(ie);
      }
    }
    iconCounter++;
  }
  return result;
};

AbstractDbOverlay.prototype.refresh = function() {
  throw new Error("Refreshing shouldn't be called");
};

AbstractDbOverlay.prototype.searchByEncodedQuery = function(originalQuery, fitBounds) {
  var query = this.decodeQuery(originalQuery);
  query.fitBounds = fitBounds;
  if (query.type === AbstractDbOverlay.QueryType.SEARCH_BY_QUERY) {
    return this.searchByQuery(query.query, query.perfect);
  } else if (query.type === AbstractDbOverlay.QueryType.SEARCH_BY_TARGET) {
    return this.searchNamesByTarget(query.target);
  } else if (query.type === AbstractDbOverlay.QueryType.SEARCH_BY_COORDINATES) {
    query.coordinates = new google.maps.Point(query.coordinates.x, query.coordinates.y);
    return this.searchByCoordinates(query);
  } else {
    throw new Error("Unknown type of query: " + query.type);
  }
};

AbstractDbOverlay.prototype.clear = function() {
  var self = this;
  return self.searchByQuery("").then(function() {
    return self.callListeners('onClear');
  });
};

/**
 * Returns true if overlay allows to get general data for element.
 */
AbstractDbOverlay.prototype.allowGeneralSearch = function() {
  return this._allowGeneralSearch;
};

AbstractDbOverlay.prototype.getName = function() {
  return this.name;
};

/**
 * Returns true if overlay allows to get data for element by search id.
 */
AbstractDbOverlay.prototype.allowSearchById = function() {
  return this._allowSearchById;
};

AbstractDbOverlay.prototype.setMap = function(map) {
  this._map = map;
};

AbstractDbOverlay.prototype.getMap = function() {
  return this._map;
};

AbstractDbOverlay.prototype.setName = function(name) {
  this.name = name;
};

AbstractDbOverlay.prototype.getName = function() {
  return this.name;
};

AbstractDbOverlay.prototype.setAllowSearchById = function(allowSearchById) {
  // configure if the overlay can contain detailed data about elements that
  // should be visualized in detailed mode of the Info Window
  if (typeof allowSearchById === "boolean") {
    this._allowSearchById = allowSearchById;
  } else if (allowSearchById === undefined) {
    this._allowSearchById = false;
  } else {
    throw new Error("Unknown type of allowSearchById: ", allowSearchById);
  }
};

AbstractDbOverlay.prototype.setAllowGeneralSearch = function(allowGeneralSearch) {
  if (typeof allowGeneralSearch === "boolean") {
    this._allowGeneralSearch = allowGeneralSearch;
  } else if (allowGeneralSearch === undefined) {
    this._allowGeneralSearch = false;
  } else {
    throw new Error("Unknown type of allowSearchById: ", allowGeneralSearch);
  }
};

AbstractDbOverlay.prototype.setIconType = function(iconType) {
  this._iconType = iconType;
};
AbstractDbOverlay.prototype.setIconStart = function(iconStart) {
  this._iconStart = iconStart;
};

AbstractDbOverlay.IconColors = [ "red", "blue", "green", "purple", "yellow", "pink", "paleblue", "brown", "orange" ];

AbstractDbOverlay.prototype.getColor = function(colorId) {
  var id = colorId + this._iconStart;
  id %= AbstractDbOverlay.IconColors.length;
  return AbstractDbOverlay.IconColors[id];
};

AbstractDbOverlay.prototype.getIcon = function(colorId, id) {
  if (id >= 100) {
    id = 1;
  }
  var color = this.getColor(colorId);
  return "marker/" + this._iconType + "/" + this._iconType + "_" + color + "_" + id + ".png";
};

AbstractDbOverlay.prototype.splitQuery = function(query, useFullName) {
  var result = [];
  if (query.indexOf(";") >= 0) {
    result = query.split(";");
  } else {
    result = query.split(",");
  }

  for (var i = 0; i < result.length; i++) {
    result[i] = result[i].trim();
  }
  if (result.length > 1 && useFullName) {
    result.push(query);
  }
  return result;
};

module.exports = AbstractDbOverlay;
