"use strict";

var Promise = require("bluebird");

var logger = require('../../logger');
var Functions = require('../../Functions');

var Comment = require('../data/Comment');
var GuiUtils = require('../../gui/leftPanel/GuiUtils');
var IdentifiedElement = require('../data/IdentifiedElement');
var ObjectWithListeners = require('../../ObjectWithListeners');
var TargettingStructure = require('../data/TargettingStructure');

/**
 * Class representing any info window in our map.
 * 
 * @param abstractMarker
 *          {@link AbstractMarker} to which this window is pointing, can be null
 */
function AbstractInfoWindow(params) {
  // call super constructor
  ObjectWithListeners.call(this);

  this.setCustomMap(params.map);
  this.setGoogleMarker(params.marker);

  this.setContent(this.createWaitingContentDiv());

  this._overlayFullView = [];

  this.registerPropertyType("overlayFullView");

  // listener called when user want to see all data about specific data overlay
  var onOverlayFullViewChanged = function(e) {
    var self = e.object;
    // first change the content of the element
    return self.update().then(function() {
      if (e.newVal) {
        var element = new IdentifiedElement({
          objectId : self.getElementId(),
          modelId : self.getCustomMap().getId(),
          type : self.getElementType()
        });
        var topMap = self.getCustomMap().getTopMap();
        return topMap.retrieveOverlayDetailDataForElement(element, self.getOverlayFullViewArray());
      }
    });

  };

  // default settings of visualizing full information about elements
  this.setOverlayFullView("drug", false);
  this.setOverlayFullView("chemical", false);
  this.setOverlayFullView("mirna", false);
  this.setOverlayFullView("search", false);
  this.setOverlayFullView("missingConnection", false);
  // only all comments should be visible from the beginning
  this.setOverlayFullView("comment", true);

  this.addPropertyChangeListener("overlayFullView", onOverlayFullViewChanged);

  this.registerListenerType("onShow");
  this.registerListenerType("onUpdate");

  this.setGuiUtils(new GuiUtils());

}

AbstractInfoWindow.prototype = Object.create(ObjectWithListeners.prototype);
AbstractInfoWindow.prototype.constructor = AbstractInfoWindow;

/**
 * Returns <code>true</code> if overlay should visualize all possible values.
 * 
 * @param overlayName
 *          name of the overlay
 * @returns <code>true</code> if overlay should visualize all possible values
 */
AbstractInfoWindow.prototype.isOverlayFullView = function(overlayName) {
  if (this._overlayFullView[overlayName] === undefined) {
    this._overlayFullView[overlayName] = false;
  }
  return this._overlayFullView[overlayName];
};

/**
 * Returns associative array with information if specific overlay should present
 * all possible results or only specified by the data searched by user.
 * 
 * @returns {Array} with information if specific overlay should present all
 *          possible results or only specified by the data searched by user
 */
AbstractInfoWindow.prototype.getOverlayFullViewArray = function() {
  return this._overlayFullView;
};

AbstractInfoWindow.prototype.setOverlayFullView = function(overlayName, value) {
  var oldVal = this._overlayFullView[overlayName];
  this._overlayFullView[overlayName] = value;
  return this.firePropertyChangeListener("overlayFullView", overlayName + "," + oldVal, value);
};

/**
 * This method checks if {@link AbstractInfoWindow} is opened.
 * 
 * @returns {Boolean} <code>true</code> if window is opened,
 *          <code>false</code> otherwise
 */
AbstractInfoWindow.prototype.isOpened = function() {
  if (this.googleInfowindow === undefined) {
    return false;
  }
  var infoWindowMap = this.googleInfowindow.getMap();
  if (infoWindowMap !== null && typeof infoWindowMap !== "undefined") {
    return true;
  } else {
    return false;
  }
};

/**
 * Opens Info Window.
 */
AbstractInfoWindow.prototype.open = function(newMarker) {
  var self = this;
  if (self.googleInfowindow === null) {
    logger.warn("Cannot open window.");
    return;
  }
  if (newMarker !== undefined) {
    self.setGoogleMarker(newMarker);
  }
  self.googleInfowindow.open(self.getCustomMap().getGoogleMap(), self.getGoogleMarker());

  return self.update().then(function() {
    return self.callListeners("onShow");
  });
};

/**
 * Sets new content of the info window.
 * 
 * @param content
 *          new content of the window
 */
AbstractInfoWindow.prototype.setContent = function(content) {
  this._content = content;
  if (this.googleInfowindow !== undefined) {
    this.googleInfowindow.setContent(content);
  }
};

/**
 * Returns content visualized in the info window.
 * 
 * @returns {String} content visualized in the info window
 */
AbstractInfoWindow.prototype.getContent = function() {
  return this._content;
};

/**
 * Creates div for an overlay data.
 * 
 * @param overlay
 *          corresponding {@link OverlayCollection}
 * @param data
 *          data taken from overlay
 * @returns string representing div for given overlay data
 */
AbstractInfoWindow.prototype.createOverlayInfoDiv = function(overlay, data) {
  var alias = this.alias;
  if (alias !== undefined) {
    if (alias.getType() !== undefined) {
      if (overlay.name === "drug") {
        if (alias.getType().toUpperCase() === "RNA" || //
        alias.getType().toUpperCase() === "PROTEIN" || //
        alias.getType().toUpperCase() === "GENE") {
          return this._createDrugInfoDiv(overlay, data);
        } else {
          return null;
        }
      } else if (overlay.name === "chemical") {
        if (this.alias.getType().toUpperCase() === "RNA" || //
        alias.getType().toUpperCase() === "PROTEIN" || //
        alias.getType().toUpperCase() === "GENE") {
          return this._createChemicalInfoDiv(overlay, data);
        } else {
          return null;
        }
      } else if (overlay.name === "mirna") {
        if (alias.getType().toUpperCase() === "RNA" || //
        alias.getType().toUpperCase() === "PROTEIN" || //
        alias.getType().toUpperCase() === "GENE") {
          return this._createMiRnaInfoDiv(overlay, data);
        } else {
          return null;
        }

      } else if (overlay.name === "comment") {
        return this._createCommentInfoDiv(overlay, data);
      } else {
        logger.warn("Unknown overlay data for AliasInfoWindow: " + overlay.name);
        return this._createDefaultInfoDiv(overlay, data);
      }
    } else {
      logger.debug(alias);
      throw new Error("Cannot customize info window. Alias type is unknown ");
    }
  } else {
    if (overlay.getName() === "comment") {
      return this._createCommentInfoDiv(overlay, data);
    } else {
      logger.debug("Cannot customize info window. Alias not defined. Overlay: " + overlay.getName());
      return null;
    }
  }
};

/**
 * Creates and returns div for drug overlay information.
 * 
 * @param data
 *          data taken from drug iverlay
 * @returns {String} with a div for drug overlay information
 */
AbstractInfoWindow.prototype._createDrugInfoDiv = function(overlay, data) {
  return this._createTargetInfoDiv({
    overlay : overlay,
    data : data,
    name : "Interacting drugs"
  });
};

/**
 * Creates and returns div for comment overlay information.
 * 
 * @param data
 *          data taken from comment iverlay
 * @returns {String} with a div for comment overlay information
 */
AbstractInfoWindow.prototype._createCommentInfoDiv = function(overlay, data) {
  if (data.length === 0 || data[0] === undefined) {
    return null;
  }
  var result = document.createElement("div");

  var titleElement = document.createElement("h3");
  titleElement.innerHTML = "Comments";
  result.appendChild(titleElement);
  for (var i = 0; i < data.length; i++) {
    var comment = data[i];
    if (comment instanceof Comment) {
      if (!comment.isRemoved()) {
        result.appendChild(document.createElement("hr"));
        var commentId = document.createElement("div");
        commentId.innerHTML = '#' + comment.getId();
        result.appendChild(commentId);
        result.appendChild(document.createElement("br"));
        var commentContent = document.createElement("div");
        commentContent.innerHTML = comment.getContent();
        result.appendChild(commentContent);
      }
    } else {
      throw new Error("Invalid comment data: " + comment);
    }
  }

  return result;
};

/**
 * Creates and returns div for unknown overlay.
 * 
 * @param overlay
 *          some {@link OverlayCollection}
 * @param data
 *          data taken from overlay
 * @returns Dom element with a div for overlay information
 */

AbstractInfoWindow.prototype._createDefaultInfoDiv = function(overlay, data) {
  var divElement = document.createElement("div");
  var count = 0;

  var titleElement = document.createElement("h3");
  var title = document.createTextNode(overlay.getName());
  titleElement.appendChild(title);
  divElement.appendChild(titleElement);
  for ( var searchId in data) {
    if (data.hasOwnProperty(searchId) && data[searchId] !== undefined && data[searchId] !== null) {
      count++;
      var resultTitleElement = document.createElement("h4");
      var resultTitle = document.createTextNode(searchId);
      resultTitleElement.appendChild(resultTitle);
      divElement.appendChild(resultTitleElement);

      var keys = Object.keys(data[searchId]);
      for (var i = 0; i < keys.length; i++) {
        var resultValElement = document.createElement("p");
        var resultVal = document.createTextNode(keys[i] + ": " + data[searchId][keys[i]]);
        resultValElement.appendChild(resultVal);
        divElement.appendChild(resultValElement);
      }
    }
  }

  if (count === 0) {
    divElement = null;
  }
  return divElement;
};

/**
 * Returns google.maps.Marker object where this info window is attached.
 * 
 * @returns google.maps.Marker object where this info window is attached
 */
AbstractInfoWindow.prototype.getGoogleMarker = function() {
  return this._marker;
};

AbstractInfoWindow.prototype.setGoogleMarker = function(marker) {
  this._marker = marker;
};

/**
 * Returns {@link AbstractCustomMap} where this window is presented.
 * 
 * @returns {@link AbstractCustomMap} where this window is presented
 */
AbstractInfoWindow.prototype.getCustomMap = function() {
  return this.customMap;
};

AbstractInfoWindow.prototype.setCustomMap = function(map) {
  if (map === undefined) {
    throw new Error("Map must be defined");
  }
  this.customMap = map;
};

/**
 * Returns html DOM object with content that should presented when waiting for
 * some data from server.
 * 
 * @returns {String} html with content that should presented when waiting for
 *          some data from server
 */
AbstractInfoWindow.prototype.createWaitingContentDiv = function() {
  var result = document.createElement("div");
  var img = document.createElement("img");
  img.src = GuiConnector.getImgPrefix() + GuiConnector.getLoadingImg();
  var message = document.createElement("h4");
  message.innerHTML = "loading...";
  result.appendChild(img);
  result.appendChild(message);
  return result;
};

/**
 * This is a generic method that updates content of the window.
 */
AbstractInfoWindow.prototype._updateContent = function() {
  var contentDiv = null;
  var self = this;

  if (!self.isOpened()) {
    return Promise.resolve();
  } else {
    self.setContent(self.createWaitingContentDiv());

    return self.createContentDiv().then(function(content) {
      contentDiv = content;
      return self.createOverlaysDiv();
    }).then(function(overlaysDiv) {
      if (overlaysDiv !== undefined && overlaysDiv !== null) {
        contentDiv.appendChild(overlaysDiv);
      }
      self.setContent(contentDiv);

      self.callListeners("onUpdate");

      return contentDiv;
    });
  }
};

/**
 * Creates and returns div with overlays content.
 * 
 * @returns {String} with html representing data taken from
 *          {@link OverlayCollection} for this window
 */
AbstractInfoWindow.prototype.createOverlaysDiv = function() {
  var self = this;
  var result = document.createElement("div");
  return this.getOverlaysData(self.getOverlayFullViewArray()).then(function(overlayData) {
    for (var i = 0; i < overlayData.length; i++) {
      var overlay = overlayData[i].overlay;
      var data = overlayData[i].data;
      var overlayInfo = self.createOverlayInfoDiv(overlay, data);
      if (overlayInfo !== null) {
        result.appendChild(overlayInfo);
      }
    }
    return result;
  });
};

/**
 * Abstract method (to be implemented by subclasses) for updating content.
 */
AbstractInfoWindow.prototype.update = function() {
  return this._updateContent();
};

AbstractInfoWindow.prototype._createTargetInfoDiv = function(params) {
  var overlay = params.overlay;
  var data = params.data;
  var name = params.name;

  var self = this;
  var result = document.createElement("div");

  var count = 0;

  var titleElement = document.createElement("h3");
  titleElement.innerHTML = name;
  result.appendChild(titleElement);
  if (overlay.allowGeneralSearch()) {
    var checkboxDiv = document.createElement("div");
    checkboxDiv.style.float = "right";

    var checkbox = document.createElement("input");
    checkbox.id = "checkbox-" + name + "-" + this.getElementType() + "-" + this.getElementId();
    checkbox.type = "checkbox";
    checkbox.checked = self.isOverlayFullView(overlay.getName());
    var checkboxClickedFunction = function() {
      return self.setOverlayFullView(overlay.getName(), this.checked).then(null, GuiConnector.alert);
    };
    checkbox.onclick = checkboxClickedFunction;

    checkboxDiv.appendChild(checkbox);

    var description = document.createElement("div");
    description.style.float = "right";
    description.innerHTML = "Show all";
    checkboxDiv.appendChild(description);
    result.appendChild(checkboxDiv);
  }
  count = 0;
  for ( var dataId in data) {
    if (data.hasOwnProperty(dataId)) {
      count++;
    }
  }

  var table = self._createTableForTargetDiv(data, overlay);

  if (count === 0 && !overlay.allowGeneralSearch() && !this.isOverlayFullView(overlay.getName())) {
    result = null;
  }
  if (result !== null) {
    result.appendChild(table);
  }
  return result;
};

AbstractInfoWindow.prototype._createTableForTargetDiv = function(data, overlay) {
  var self = this;
  var table = document.createElement("table");
  table.className = "minerva-window-drug-table";
  var header = document.createElement("tr");
  var headerCol = document.createElement("th");
  headerCol.innerHTML = "Name";
  header.appendChild(headerCol);
  headerCol = document.createElement("th");
  headerCol.innerHTML = "References";
  header.appendChild(headerCol);

  var cell;
  table.appendChild(header);
  var row;

  var onclick = function() {
    // ';' enforces single query (in case there are ',' characters in the name)
    return overlay.searchByQuery(this.innerHTML + ";");
  };

  var count = 0;
  for ( var searchId in data) {
    if (data.hasOwnProperty(searchId)) {

      row = document.createElement("tr");
      var nameContent = searchId;
      var annotations = [];
      if (typeof data[searchId] === "string") {
        nameContent = data[searchId];
      } else if (data[searchId] instanceof TargettingStructure) {
        nameContent = data[searchId].getName();
        var targets = data[searchId].getTargetsForIdentifiedElement(self.getIdentifiedElement());
        for (var i = 0; i < targets.length; i++) {
          var references = targets[i].getReferences();
          for (var j = 0; j < references.length; j++) {
            annotations.push(references[j]);
          }
        }
      }
      var link = Functions.createElement({
        type : "a",
        onclick : onclick,
        href : "#",
        content : nameContent,
      });

      var nameTd = Functions.createElement({
        type : "td",
      });
      nameTd.appendChild(link);
      row.appendChild(nameTd);

      var referencesCell = Functions.createElement({
        type : "td",
      });
      referencesCell.appendChild(self.getGuiUtils().createAnnotationList(annotations));

      row.appendChild(referencesCell);

      table.appendChild(row);
      count++;
    }
  }

  if (self.isOverlayFullView(overlay.getName()) && count === 0) {
    row = document.createElement("tr");
    cell = document.createElement("td");
    cell.colSpan = "2";
    cell.innerHTML = "No results available";
    row.appendChild(cell);
    table.appendChild(row);
  }

  if (!self.isOverlayFullView(overlay.getName()) && count === 0 && overlay.allowGeneralSearch()) {
    row = document.createElement("tr");
    cell = document.createElement("td");
    cell.colSpan = "2";
    cell.innerHTML = "Search for available targets";
    row.appendChild(cell);
    table.appendChild(row);
  }
  return table;
};

AbstractInfoWindow.prototype.init = function() {
  var self = this;
  return ServerConnector.getConfiguration().then(function(configuration) {
    self.getGuiUtils().setConfiguration(configuration);
    self.getGuiUtils().setMap(self.getCustomMap());
  });
};
AbstractInfoWindow.prototype.setGuiUtils = function(guiUtils) {
  this._guiUtils = guiUtils;
};
AbstractInfoWindow.prototype.getGuiUtils = function() {
  return this._guiUtils;
};

/**
 * Creates and returns DOM div for chemical overlay information.
 * 
 * @param data
 *          data taken from chemical overlay
 * @returns DOM element with a div for comment overlay information
 */
AbstractInfoWindow.prototype._createChemicalInfoDiv = function(overlay, data) {
  return this._createTargetInfoDiv({
    overlay : overlay,
    data : data,
    name : "Interacting chemicals",
  });
};

/**
 * Creates and returns DOM div for mi rna overlay information.
 * 
 * @param data
 *          data taken from mi rna overlay
 * @returns DOM element with a div for comment overlay information
 */
AbstractInfoWindow.prototype._createMiRnaInfoDiv = function(overlay, data) {
  return this._createTargetInfoDiv({
    overlay : overlay,
    data : data,
    name : "Interacting Micro RNAs"
  });
};

/**
 * Abstract method returning identifier of the object for which this window was
 * created.
 */
AbstractInfoWindow.prototype.getElementId = function() {
  throw new Error("Not implemented");
};

/**
 * Abstract method returning type of the object for which this window was
 * created.
 */
AbstractInfoWindow.prototype.getElementType = function() {
  throw new Error("Not implemented");
};

AbstractInfoWindow.prototype.getIdentifiedElement = function() {
  var self = this;
  return new IdentifiedElement({
    id : self.getElementId(),
    type : self.getElementType(),
    modelId : self.getCustomMap().getId()
  });
};

module.exports = AbstractInfoWindow;
