"use strict";

var Promise = require("bluebird");

require("../mocha-config.js");

var AliasMarker = require('../../../main/js/map/marker/AliasMarker');
var AliasSurface = require('../../../main/js/map/surface/AliasSurface');
var Comment = require('../../../main/js/map/data/Comment');
var CustomMap = require('../../../main/js/map/CustomMap');
var IdentifiedElement = require('../../../main/js/map/data/IdentifiedElement');
var MapContextMenu = require('../../../main/js/gui/MapContextMenu');
var PointData = require('../../../main/js/map/data/PointData');
var PointMarker = require('../../../main/js/map/marker/PointMarker');
var ReactionMarker = require('../../../main/js/map/marker/ReactionMarker');
var ReactionSurface = require('../../../main/js/map/surface/ReactionSurface');
var SelectionContextMenu = require('../../../main/js/gui/SelectionContextMenu');

var logger = require('./../logger');

var chai = require('chai');
var assert = chai.assert;

describe('CustomMap', function() {
  describe("constructor", function() {
    it("default", function() {
      var options = helper.createCustomMapOptions();
      var map = new CustomMap(options);
      assert.ok(map);
    });

    it("with submaps", function() {
      var options = helper.createCustomMapOptions();

      options.getProject().getModel().addSubmodel(helper.createModel());

      var map = new CustomMap(options);
      assert.ok(map);
    });

    it("with session data pointing to not existing overlay", function() {
      var options = helper.createCustomMapOptions();

      ServerConnector.getSessionData(options.getProject()).setSelectedBackgroundOverlay("-1");

      var map = new CustomMap(options);
      assert.ok(map);
    });
  });

  it("getSubmapById", function() {
    var map = helper.createCustomMap();
    assert.ok(map.getSubmapById(map.getId()));
  });

  it("getSubmapById (invalid)", function() {
    var map = helper.createCustomMap();
    assert.equal(map.getSubmapById(-1), null);
  });

  it("getSubmapById (string id)", function() {
    var map = helper.createCustomMap();
    assert.ok(map.getSubmapById(map.getId() + ""));
  });

  describe("openDataOverlay", function() {
    it("for not existing id", function() {
      var map = helper.createCustomMap();
      try {
        map.openDataOverlay(-1);
        assert.ok(false);
      } catch (exception) {
        assert.ok(exception.message.indexOf("You have no privileges") >= 0);
      }
    });

    it("for int id", function() {
      var options = helper.createCustomMapOptions();
      var model = options.getProject().getModel();
      var layout = model.getLayouts()[0];
      var map = new CustomMap(options);
      map.openDataOverlay(layout.getId());
      assert.equal(logger.getErrors().length, 0);
    });

    it("for overlay object", function() {
      var options = helper.createCustomMapOptions();
      var model = options.getProject().getModel();
      var layout = model.getLayouts()[0];
      var map = new CustomMap(options);
      map.openDataOverlay(layout);
      assert.equal(logger.getErrors().length, 0);
    });

    it("for string id", function() {
      var options = helper.createCustomMapOptions();
      var model = options.getProject().getModel();
      var layout = model.getLayouts()[0];
      var map = new CustomMap(options);
      map.openDataOverlay(layout.getId() + "");
      assert.equal(logger.getErrors().length, 0);
    });

    it("with background overlay", function() {
      var map = helper.createCustomMap();
      var layout = helper.createLayout();
      layout.setInitialized(true);
      layout.setInputDataAvailable(false);
      map.getModel().addLayout(layout);

      return map.openDataOverlay(layout.getId()).then(function() {
        var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
        assert.equal(0, vLayouts.length);
      });
    });

    it("simple", function() {
      var map = helper.createCustomMap();
      var alias = helper.createAlias();
      alias.setModelId(map.getId());
      map.getModel().addAlias(alias);

      var reaction = helper.createReaction();
      map.getModel().addReaction(reaction);

      var layout = helper.createLayout();
      layout.setInputDataAvailable(true);
      layout.setInitialized(true);
      var layoutAlias = helper.createLayoutAlias();
      layoutAlias.setId(alias.getId());
      layoutAlias.setModelId(map.getId());
      layout.addAlias(layoutAlias);

      var layoutReaction = helper.createLayoutReaction();
      layoutReaction.setId(reaction.getId());
      layoutReaction.setModelId(map.getId());
      layout.addReaction(layoutReaction);

      map.getModel().addLayout(layout);

      return map.openDataOverlay(layout.getId()).then(function() {
        return map._showSelectedLayout(layout.getId(), 0, 1);
      }).then(function() {
        var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
        assert.equal(1, vLayouts.length);
        assert.equal(layout.getId(), vLayouts[0]);

        assert.equal(2, map.selectedLayoutOverlays[layout.getId()].length);
        assert.ok(map.selectedLayoutOverlays[layout.getId()][0] instanceof AliasSurface);
        assert.ok(map.selectedLayoutOverlays[layout.getId()][1] instanceof ReactionSurface);

        // now hide the layout
        map._hideSelectedLayout(layout.getId());
        assert.ok(map.selectedLayoutOverlays[layout.getId()]);
        assert.equal(0, map.selectedLayoutOverlays[layout.getId()].length);
      });

    });

    it("with submaps", function() {
      var projectId = "complex_model_with_submaps";
      helper.setUrl("http://test/?id=" + projectId);
      GuiConnector.getParams["id"] = projectId;
      var customMap;
      var emptySubmodelId = 16730;
      var filledSubmodelId = 16731;
      var overlayId = 18083;

      return ServerConnector.getProject(projectId).then(function(project) {
        var options = helper.createCustomMapOptions(project);
        customMap = new CustomMap(options);

        return customMap.openSubmap(emptySubmodelId);
      }).then(function() {
        return customMap.openSubmap(filledSubmodelId);
      }).then(function() {

        var emptySubmap = customMap.getSubmapById(emptySubmodelId);
        var filledSubmap = customMap.getSubmapById(filledSubmodelId);

        return customMap.openDataOverlay(overlayId).then(function() {

          assert.equal(1, customMap.selectedLayoutOverlays[overlayId].length);
          assert.equal(1, filledSubmap.selectedLayoutOverlays[overlayId].length);
          assert.equal(0, emptySubmap.selectedLayoutOverlays[overlayId].length);

          // now hide the layout
          customMap.hideSelectedLayout(overlayId);
          assert.equal(0, customMap.selectedLayoutOverlays[overlayId].length);
          assert.equal(0, filledSubmap.selectedLayoutOverlays[overlayId].length);
          assert.equal(0, emptySubmap.selectedLayoutOverlays[overlayId].length);
        });

      });
    });

  });

  it("removeSelectedLayout", function() {
    var map = helper.createCustomMap();
    var layout = helper.createLayout();
    layout.setInitialized(true);
    layout.setInputDataAvailable(true);
    map.getModel().addLayout(layout);

    return map.openDataOverlay(layout.getId()).then(function() {
      return map.removeSelectedLayout(layout.getId());
    }).then(function() {
      var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
      assert.equal(0, vLayouts.length);
    });
  });

  describe("openInfoWindowForAlias", function() {
    it("default", function() {
      var map = helper.createCustomMap();
      var layout = helper.createLayout();
      var alias = helper.createAlias();
      alias.setModelId(map.getId());
      var layoutAlias = helper.createLayoutAlias();
      layoutAlias.setId(alias.getId());

      layout.addAlias(layoutAlias);

      // create layout
      map.getModel().addLayout(layout);
      map.getModel().addAlias(alias);

      return map.openInfoWindowForAlias(alias).then(function() {
        assert.ok(map.getAliasInfoWindowById(alias.getId()));
        assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());
      });
    });

    it("for incomplete alias", function() {
      var map, alias;
      return ServerConnector.getProject().then(function(project) {
        var options = helper.createCustomMapOptions(project);
        map = new CustomMap(options);

        return map.getModel().getAliasById(329171, false);
      }).then(function(result) {
        alias = result;
        assert.notOk(alias.isComplete());
        return map.openInfoWindowForAlias(alias);
      }).then(function() {
        assert.ok(map.getAliasInfoWindowById(alias.getId()));
        assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());
      });
    });
  });

  it("getSearchAutocomplete", function() {
    var map = helper.createCustomMap();

    var t = map.getSearchAutocomplete("s");
    assert.ok(t);
    assert.equal(t.length, 0);
  });

  it("getSearchAutocomplete 2", function() {
    var map = helper.createCustomMap();

    return map.refreshSearchAutocomplete("s").then(function() {
      var t = map.getSearchAutocomplete("s");
      assert.ok(t);
      assert.ok(t.length > 0);
    });
  });

  describe("renderOverlayCollection", function() {
    it("for alias", function() {
      var map = helper.createCustomMap();
      var reaction = helper.createReaction();
      var alias = helper.createAlias();
      map.getModel().addAlias(alias);
      map.getModel().addReaction(reaction);

      var oc = helper.createDbOverlay(map);

      oc.getIdentifiedElements = function() {
        return Promise.resolve([ new IdentifiedElement({
          objectId : alias.getId(),
          icon : "empty.png",
          modelId : map.getId(),
          type : "Alias"
        }) ]);
      };
      return map.renderOverlayCollection({
        overlayCollection : oc
      }).then(function() {
        var aliasMarkerCount = 0;
        var markers = oc.markers["ALIAS"];
        for ( var id in markers) {
          if (markers.hasOwnProperty(id)) {
            aliasMarkerCount++;
          }
        }
        assert.equal(1, aliasMarkerCount);
      });
    });

    it("alias re-rendering with different icon", function() {
      var map = helper.createCustomMap();
      var reaction = helper.createReaction(map);
      var alias = helper.createAlias(map);
      map.getModel().addAlias(alias);
      map.getModel().addReaction(reaction);

      var marker;
      var oc = helper.createDbOverlay(map);

      oc.getIdentifiedElements = function() {
        return Promise.resolve([ new IdentifiedElement({
          objectId : alias.getId(),
          icon : "empty.png",
          modelId : map.getId(),
          type : "Alias"
        }), new IdentifiedElement(reaction) ]);
      };
      return map.renderOverlayCollection({
        overlayCollection : oc
      }).then(function() {
        marker = oc.markers["ALIAS"][alias.getId()];
        oc.getIdentifiedElements = function() {
          return Promise.resolve([ new IdentifiedElement({
            objectId : alias.getId(),
            icon : "new_icon.png",
            modelId : map.getId(),
            type : "Alias"
          }) ]);
        };
        assert.ok(oc.mapOverlays["REACTION"][reaction.getId()]);

        return map.renderOverlayCollection({
          overlayCollection : oc
        });
      }).then(function() {
        assert.equal(marker.getIcon(), "new_icon.png");

        assert.notOk(oc.mapOverlays["REACTION"][reaction.getId()]);
      });
    });

    it("for set of aliases", function() {
      var map = helper.createCustomMap();

      var oc = helper.createDbOverlay(map);

      var identifiedElements = [];
      for (var i = 0; i < 3; i++) {
        var alias = helper.createAlias(map);
        map.getModel().addAlias(alias);
        var ie = helper.createIdentifiedElement(alias);
        ie.setIcon("");
        identifiedElements.push(ie);
      }

      oc.getIdentifiedElements = function() {
        return Promise.resolve(identifiedElements);
      };

      return map.renderOverlayCollection({
        overlayCollection : oc
      }).then(function() {
        var aliasMarkerCount = 0;
        var markers = oc.markers["ALIAS"];
        for ( var id in markers) {
          if (markers.hasOwnProperty(id)) {
            aliasMarkerCount++;
          }
        }

        assert.equal(aliasMarkerCount, 3);
      });
    });

    it("for point", function() {
      var map = helper.createCustomMap();

      var oc = helper.createDbOverlay(map);

      var javaObj = {
        objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
        modelId : map.getId(),
        type : "POINT",
        icon : "marker/empty.png"
      };

      oc.getIdentifiedElements = function() {
        return Promise.resolve([ new IdentifiedElement(javaObj) ]);
      };

      return map.renderOverlayCollection({
        overlayCollection : oc
      }).then(function() {
        var markerCount = 0;
        var markers = oc.markers["POINT"];
        for ( var id in markers) {
          if (markers.hasOwnProperty(id)) {
            markerCount++;
          }
        }
        assert.equal(1, markerCount);
      });
    });

    it("for reaction", function() {
      var map = helper.createCustomMap();
      var reaction = helper.createReaction();
      map.getModel().addReaction(reaction);

      var oc = helper.createDbOverlay(map);

      var javaObj = {
        objectId : reaction.getId(),
        modelId : map.getId(),
        type : "Reaction",
      };

      oc.getIdentifiedElements = function() {
        return Promise.resolve([ new IdentifiedElement(javaObj) ]);
      };

      return map.renderOverlayCollection({
        overlayCollection : oc
      }).then(function() {
        var markerCount = 0;
        var markers = oc.mapOverlays["REACTION"];
        for ( var id in markers) {
          if (markers.hasOwnProperty(id)) {
            markerCount++;
          }
        }
        assert.equal(1, markerCount);
      });
    });
  });

  it("clearDbOverlays", function() {
    var map = helper.createCustomMap();

    var oc = helper.createDbOverlay(map);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };
    var searchResults = [ new IdentifiedElement(javaObj) ];

    oc.getIdentifiedElements = function() {
      return Promise.resolve(searchResults);
    };
    oc.clear = function() {
      searchResults = [];
      return this.callListeners("onSearch", searchResults);
    };

    return map.renderOverlayCollection({
      overlayCollection : oc
    }).then(function() {

      return map.clearDbOverlays();
    }).then(function() {
      var markerCount = 0;
      for ( var id in oc.pointMarkers) {
        if (oc.pointMarkers.hasOwnProperty(id)) {
          markerCount++;
        }
      }

      assert.equal(0, markerCount);
    });

  });

  it("getInfoWindowForIdentifiedElement ( reaction)", function() {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction(map);
    map.getModel().addReaction(reaction);

    var ie = helper.createIdentifiedElement(reaction);

    var infoWindow = map.getInfoWindowForIdentifiedElement(ie);

    assert.equal(null, infoWindow);
    return map._openInfoWindowForReaction(reaction).then(function() {
      infoWindow = map.getInfoWindowForIdentifiedElement(ie);

      assert.ok(infoWindow);
    });

  });

  it("right click on map", function() {
    var map = helper.createCustomMap();
    map.setContextMenu(new MapContextMenu({
      customMap : map,
      element : testDiv
    }));

    var mev = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    google.maps.event.trigger(map.getGoogleMap(), 'rightclick', mev);
    assert.equal(map.getId(), map.getActiveSubmapId());

  });

  it("left click on map", function() {
    var map;
    var searchOverlay;
    return ServerConnector.getProject().then(function(project) {
      map = helper.createCustomMap(project);
      searchOverlay = helper.createSearchDbOverlay(map);

      var mev = {
        stop : null,
        latLng : new google.maps.LatLng(82.32061703407554, -167.25586206896548),
      };
      map.getGoogleMap().setZoom(4);
      // latLng : new google.maps.LatLng(82.40238643645326, -148.44758620689652)

      assert.notOk(map.getActiveSubmapId());
      return google.maps.event.trigger(map.getGoogleMap(), 'click', mev);

    }).then(function() {
      assert.equal(map.getId(), map.getActiveSubmapId());
      assert.ok(searchOverlay.markers["ALIAS"][329171]);
    });

  });

  it("left click on reaction", function() {
    var map;
    var searchOverlay;
    return ServerConnector.getProject().then(function(project) {
      map = helper.createCustomMap(project);
      searchOverlay = helper.createSearchDbOverlay(map);

      var mev = {
        stop : null,
        latLng : new google.maps.LatLng(82.40238643645326, -148.44758620689652)
      };
      map.getGoogleMap().setZoom(4);

      assert.notOk(map.getActiveSubmapId());
      return google.maps.event.trigger(map.getGoogleMap(), 'click', mev);
    }).then(function() {
      assert.equal(map.getId(), map.getActiveSubmapId());
      assert.ok(searchOverlay.mapOverlays["REACTION"][153521]);
      assert.ok(searchOverlay.mapOverlays["REACTION"][153521].isShown());
      assert.ok(searchOverlay.markers["ALIAS"][329165]);
    });
  });

  it("create polygon through drawingManager", function() {
    var map = helper.createCustomMap();

    map.setSelectionContextMenu(new SelectionContextMenu({
      customMap : map,
      element : testDiv
    }));

    map.turnOnDrawing();

    var triangleCoords = [ new google.maps.LatLng(25.774, -80.190), new google.maps.LatLng(18.466, -66.118),
        new google.maps.LatLng(32.321, -64.757), new google.maps.LatLng(25.774, -80.190), ];

    // Construct the polygon.
    var bermudaTriangle = new google.maps.Polygon({
      paths : triangleCoords,
      strokeColor : '#FF0000',
      strokeOpacity : 0.8,
      strokeWeight : 2,
      fillColor : '#FF0000',
      fillOpacity : 0.35
    });
    bermudaTriangle.setMap(map.getGoogleMap());

    var eventParam = {
      type : google.maps.drawing.OverlayType.POLYGON,
      overlay : bermudaTriangle,
    };

    // check creation complete behaviour
    assert.ok(map.isDrawingOn());
    assert.equal(google.maps.drawing.OverlayType.POLYGON, map._drawingManager.getDrawingMode());
    google.maps.event.trigger(map._drawingManager, 'overlaycomplete', eventParam);
    assert.ok(map.isDrawingOn());
    assert.equal(null, map._drawingManager.getDrawingMode());

    // and now check how it behaves when we wiught click on the polygon
    var clickEventParam = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getSelectedPolygon());

    assert.notOk(map.getActiveSubmapId());
    assert.notOk(map.getSelectedArea());
    google.maps.event.trigger(bermudaTriangle, 'rightclick', clickEventParam);
    assert.ok(map.getSelectedArea());
    assert.equal(map.getId(), map.getActiveSubmapId());

    assert.ok(map.getSelectedPolygon());

  });

  it("getAliasVisibleLayoutsData", function() {
    var mockObject = helper.createCustomMap();

    var alias = helper.createAlias();
    mockObject.getModel().addAlias(alias);

    var layout = helper.createLayout();
    layout.setInputDataAvailable(true);
    layout.setInitialized(true);
    mockObject.getModel().addLayout(layout);

    return mockObject.openDataOverlay(layout.getId()).then(function() {
      return mockObject.getAliasVisibleLayoutsData(alias.getId());
    }).then(function(layoutAliases) {
      assert.equal(layoutAliases.length, 1);
    });
  });

  it("changed coords in map", function() {
    var map = helper.createCustomMap();
    var oldCenter = map.getGoogleMap().getCenter();
    var newCenter = new google.maps.LatLng(3, 87);
    map.getGoogleMap().setCenter(newCenter);
    google.maps.event.trigger(map.getGoogleMap(), "center_changed");

    var center = ServerConnector.getSessionData(map.getProject()).getCenter(map.getModel());
    assert.ok(center !== oldCenter);
    assert.ok(center instanceof google.maps.Point);
  });

  it("refreshComments", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    var commentsOverlay = helper.createCommentDbOverlay(map);

    ServerConnector.getSessionData().setShowComments(true);
    return map.refreshComments().then(function() {
      assert.notOk(commentsOverlay.markers["POINT"]['(241.01,372.35)']);
      assert.ok(commentsOverlay.markers["POINT"]['(643.96,144.09)']);
      assert.notOk(commentsOverlay.markers["POINT"]['(216.65,370.00)']);
      assert.notOk(commentsOverlay.markers["POINT"]['unkId']);
    });
  });

  it("hide comments", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    var commentsOverlay = helper.createCommentDbOverlay(map);

    ServerConnector.getSessionData().setShowComments(true);
    return map.refreshComments().then(function() {
      ServerConnector.getSessionData().setShowComments(false);
      return map.refreshComments();
    }).then(function() {
      assert.notOk(commentsOverlay.markers["POINT"]['(241.01, 372.35)']);
    });
  });

  it("openCommentDialog", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    map.setActiveSubmapId(15781);
    map.setActiveSubmapClickCoordinates(new google.maps.Point(2, 12));
    return map.openCommentDialog().then(function() {
      var types = map.getCommentDialog().getTypes();
      assert.equal(types.length, 6);
      var selected = map.getCommentDialog().getSelectedType();
      assert.ok(selected === "<General>");

      map.getCommentDialog().setSelectedType(1);
      selected = map.getCommentDialog().getSelectedType();
      assert.notOk(selected === "<General>");

      map.getCommentDialog().setSelectedType(2);
      selected = map.getCommentDialog().getSelectedType();
      assert.notOk(selected === "<General>");
    }).then(function() {
      map.getCommentDialog().destroy();
    });
  });

  it("addComment", function() {
    var map;
    return ServerConnector.getProject().then(function(project) {
      map = helper.createCustomMap(project);
      helper.createCommentDbOverlay(map);
      map.setActiveSubmapId(map.getId());
      map.setActiveSubmapClickCoordinates(new google.maps.Point(2, 12));
      return map.openCommentDialog();
    }).then(function() {
      return map.getCommentDialog().addComment();
    }).then(function() {
      map.getCommentDialog().destroy();
    });
  });

  it("addComment for protein", function() {
    var map;
    return ServerConnector.getProject().then(function(project) {
      map = helper.createCustomMap(project);
      helper.createCommentDbOverlay(map);
      map.setActiveSubmapId(map.getId());
      map.setActiveSubmapClickCoordinates(new google.maps.Point(2, 12));
      return map.openCommentDialog();
    }).then(function() {
      var dialog = map.getCommentDialog();
      dialog.setSelectedType(1);
      return dialog.addComment();
    }).then(function() {
      map.getCommentDialog().destroy();
    });
  });

  it("retrieveOverlayDetailDataForElement for comment", function() {
    var map = helper.createCustomMap();
    helper.createCommentDbOverlay(map);

    var alias = helper.createAlias(map);
    alias.setId(329158);
    map.getModel().addAlias(alias);

    var ie = new IdentifiedElement(alias);

    return map.retrieveOverlayDetailDataForElement(ie, {
      comment : true
    }).then(function(details) {
      assert.ok(details);
      assert.equal(details.length, 1);
      assert.equal(details[0].length, 1);
      assert.ok(details[0][0] instanceof Comment);
    });
  });

  it("getOverlayDataForIdentifiedElement", function() {
    var map = helper.createCustomMap();
    var commentOverlay = helper.createCommentDbOverlay(map);

    var alias = helper.createAlias(map);
    alias.setId(329158);
    map.getModel().addAlias(alias);

    var ie = new IdentifiedElement(alias);

    return map.getOverlayDataForIdentifiedElement(ie, {
      comment : true
    }).then(function(details) {
      assert.equal(details.length, 1);

      var overlayData = details[0];
      assert.equal(overlayData.overlay, commentOverlay);
      assert.ok(overlayData.data);
      assert.equal(overlayData.data.length, 1);
      assert.ok(overlayData.data[0] instanceof Comment);
    });
  });

  it("openSubmap", function() {
    var options = helper.createCustomMapOptions();

    var submodel = helper.createModel();
    options.getProject().getModel().addSubmodel(submodel);

    var map = new CustomMap(options);

    map.openSubmap(submodel.getId());
  });

  it("setCenter", function() {
    var options = helper.createCustomMapOptions();

    var map = new CustomMap(options);

    map.setCenter(new google.maps.LatLng(10, 20));
    assert.ok(ServerConnector.getSessionData().getCenter(map.getModel()));
  });

  it("setCenter on submap", function() {
    var options = helper.createCustomMapOptions();

    var submodel = helper.createModel();
    options.getProject().getModel().addSubmodel(submodel);

    var map = new CustomMap(options);
    map.openSubmap(submodel.getId());
    map.setCenter(new google.maps.LatLng(10, 20));
    assert.ok(ServerConnector.getSessionData().getCenter(submodel));
  });

  it("setZoom", function() {
    var options = helper.createCustomMapOptions();

    var map = new CustomMap(options);

    map.setZoom(3);
    assert.equal(3, ServerConnector.getSessionData().getZoomLevel(map.getModel()));
  });

  it("removeSelection when invalid model selected", function() {
    var options = helper.createCustomMapOptions();

    var map = new CustomMap(options);
    map.setActiveSubmapId(-1);

    try {
      map.removeSelection();
      assert.ok(false);
    } catch (e) {
      assert.ok(e.message.indexOf("Cannot find submap") >= 0);
    }
  });

  it("removeSelection", function() {
    var options = helper.createCustomMapOptions();

    var map = new CustomMap(options);
    map.setActiveSubmapId(map.getId());
    map.setSelectedArea(new google.maps.Rectangle({}));

    map.removeSelection();
    assert.equal(map.getSelectedArea(), null);
  });

  describe("createMarkerForDbOverlay", function() {
    it("update marker", function() {
      var map;
      var marker1, marker2;
      var element = new IdentifiedElement({
        id : 329159,
        modelId : 15781,
        type : "ALIAS"
      });
      var overlay;
      return ServerConnector.getProject().then(function(project) {
        var options = helper.createCustomMapOptions(project);
        map = new CustomMap(options);
        overlay = helper.createDbOverlay(map);
        return map.createMarkerForDbOverlay(element, overlay);
      }).then(function(result) {
        marker1 = result;
        element.setIcon("another.png");
        return map.createMarkerForDbOverlay(element, overlay);
      }).then(function(result) {
        marker2 = result;
        assert.equal(marker1, marker2);
      });
    });

    it("update surface", function() {
      var map;
      var marker1, marker2;
      var element = new IdentifiedElement({
        id : 329159,
        modelId : 15781,
        type : "ALIAS"
      });
      var overlay;
      return ServerConnector.getProject().then(function(project) {
        var options = helper.createCustomMapOptions(project);
        map = new CustomMap(options);
        overlay = helper.createDbOverlay(map);
        return map.createSurfaceForDbOverlay(element, overlay);
      }).then(function(result) {
        marker1 = result;
        element.setColor("another.png");
        return map.createSurfaceForDbOverlay(element, overlay);
      }).then(function(result) {
        marker2 = result;
        assert.equal(marker1, marker2);
      });
    });

    it("reaction marker", function() {
      var map;
      var element = new IdentifiedElement({
        id : 153508,
        modelId : 15781,
        type : "REACTION"
      });
      var overlay;
      return ServerConnector.getProject().then(function(project) {
        var options = helper.createCustomMapOptions(project);
        map = new CustomMap(options);
        overlay = helper.createDbOverlay(map);
        return map.createMarkerForDbOverlay(element, overlay);
      }).then(function(result) {
        assert.ok(result.getReactionData());
      });
    });

  });

  describe("_openInfoWindowForIdentifiedElement", function() {
    it("for AliasMarker", function() {
      var map;
      var alias, marker;
      return ServerConnector.getProject().then(function(project) {
        map = helper.createCustomMap(project);
        return map.getModel().getAliasById(329171);
      }).then(function(result) {
        alias = result;

        var identifiedElement = new IdentifiedElement(alias);
        identifiedElement.setIcon("empty.png");

        marker = new AliasMarker({
          element : identifiedElement,
          map : map
        });

        return marker.init();
      }).then(function() {
        assert.equal(null, map.getAliasInfoWindowById(alias.getId()));
        return map._openInfoWindowForIdentifiedElement(marker).then(function() {
          assert.ok(map.getAliasInfoWindowById(alias.getId()));
        });
      });

    });
    it("for ReactionMarker", function() {
      var map;
      var reaction, marker;
      return ServerConnector.getProject().then(function(project) {
        map = helper.createCustomMap(project);
        return map.getModel().getReactionById(153510);
      }).then(function(result) {
        reaction = result;

        marker = new ReactionMarker({
          element : new IdentifiedElement(reaction),
          map : map
        });
        return marker.init();
      }).then(function() {
        assert.equal(null, map.getReactionInfoWindowById(reaction.getId()));
        return map._openInfoWindowForIdentifiedElement(marker);
      }).then(function() {
        assert.ok(map.getReactionInfoWindowById(reaction.getId()));
      });

    });
    it("for PointMarker", function() {

      var mockObject = helper.createCustomMap();

      mockObject.getOverlayDataForPoint = function() {
        return Promise.resolve([]);
      };

      var point = new google.maps.Point(2, 3.45);
      var pointData = new PointData(point, mockObject.getId());

      var pointMarker = new PointMarker({
        element : new IdentifiedElement(pointData),
        map : mockObject
      });

      return pointMarker.init().then(function() {
        assert.equal(null, mockObject.getPointInfoWindowById(pointData.getId()));
        return mockObject._openInfoWindowForIdentifiedElement(pointMarker.getIdentifiedElement());
      }).then(function() {
        assert.ok(mockObject.getPointInfoWindowById(pointData.getId()));
      });
    });
  });

});
